@@ -41,6 +41,7 @@ from _browser import Browser, RestfulHttp
 from _json import DatetimeJSONEncoder
 from errors import HTTPError
 
+missing = object()
 
 class HeaderDictionary:
     """A dictionary that bridges httplib2's and wadllib's expectations.
@@ -62,7 +63,6 @@ class HeaderDictionary:
 
     def __getitem__(self, key):
         """Retrieve a value, converting the key to lowercase."""
-        missing = object()
         value = self.get(key, missing)
         if value is missing:
             raise KeyError(key)
@@ -333,6 +333,10 @@ class Resource(RestfulBase):
             self.__dict__['_wadl_resource'] = self._wadl_resource.bind(
                 representation, self.JSON_MEDIA_TYPE)
 
+    def __ne__(self, other):
+        """Inequality operator."""
+        return not self == other
+
 
 class HostedFile(Resource):
     """A resource representing a file managed by a lazr.restful service."""
@@ -352,6 +356,18 @@ class HostedFile(Resource):
         """HostedFile objects define no web service parameters."""
         return []
 
+    def __eq__(self, other):
+        """Equality comparison.
+
+        Two hosted files are the same if they have the same URL.
+
+        There is no need to check the contents because the only way to
+        retrieve or modify the hosted file contents is to open a
+        filehandle, which goes direct to the server.
+        """
+        return (other is not None and
+                self._wadl_resource.url == other._wadl_resource.url)
+
 
 class ServiceRoot(Resource):
     """Entry point to the service. Subclass this for a service-specific client.
@@ -363,19 +379,24 @@ class ServiceRoot(Resource):
     # instantiating resources of a certain WADL type.
     RESOURCE_TYPE_CLASSES = {'HostedFile': HostedFile}
 
-    def __init__(self, credentials, service_root, cache=None,
-                 timeout=None, proxy_info=None):
+    def __init__(self, authorizer, service_root, cache=None,
+                 timeout=None, proxy_info=None, version=None):
         """Root access to a lazr.restful API.
 
         :param credentials: The credentials used to access the service.
         :param service_root: The URL to the root of the web service.
         :type service_root: string
         """
+        if version is not None:
+            if service_root[-1] != '/':
+                service_root += '/'
+            service_root += str(version)
+            if service_root[-1] != '/':
+                service_root += '/'
         self._root_uri = URI(service_root)
-        self.credentials = credentials
         # Get the WADL definition.
         self._browser = Browser(
-            self, self.credentials, cache, timeout, proxy_info)
+            self, authorizer, cache, timeout, proxy_info)
         self._wadl = self._browser.get_wadl_application(self._root_uri)
 
         # Get the root resource.
@@ -383,15 +404,16 @@ class ServiceRoot(Resource):
         bound_root = root_resource.bind(
             self._browser.get(root_resource), 'application/json')
         super(ServiceRoot, self).__init__(None, bound_root)
+        self.credentials = authorizer
 
-    def httpFactory(self, credentials, cache, timeout, proxy_info):
-        return RestfulHttp(credentials, cache, timeout, proxy_info)
+    def httpFactory(self, authorizer, cache, timeout, proxy_info):
+        return RestfulHttp(authorizer, cache, timeout, proxy_info)
 
     def load(self, url):
         """Load a resource given its URL."""
         document = self._browser.get(url)
         try:
-            representation = simplejson.loads(document)
+            representation = simplejson.loads(unicode(document))
         except ValueError:
             raise ValueError("%s doesn't serve a JSON document." % url)
         type_link = representation.get("resource_type_link")
@@ -495,13 +517,13 @@ class NamedOperation(RestfulBase):
             # The operation returned a document with nothing
             # special about it.
             if content_type == self.JSON_MEDIA_TYPE:
-                return simplejson.loads(content)
+                return simplejson.loads(unicode(content))
             # We don't know how to process the content.
             return content
 
         # The operation returned a representation of some
         # resource. Instantiate a Resource object for it.
-        document = simplejson.loads(content)
+        document = simplejson.loads(unicode(content))
         if document is None:
             # The operation returned a null value.
             return document
@@ -577,6 +599,19 @@ class Entry(Resource):
                                  (self.__class__.__name__, name))
         self._dirty_attributes[name] = value
 
+    def __eq__(self, other):
+        """Equality operator.
+
+        Two entries are the same if their self_link and http_etag
+        attributes are the same, and if their dirty attribute dicts
+        contain the same values.
+        """
+        return (
+            other is not None and
+            self.self_link == other.self_link and
+            self.http_etag == other.http_etag and
+            self._dirty_attributes == other._dirty_attributes)
+
     def lp_refresh(self, new_url=None):
         """Update this resource's representation."""
         etag = getattr(self, 'http_etag', None)
@@ -613,7 +648,7 @@ class Entry(Resource):
         if response.status == 209 and content_type == self.JSON_MEDIA_TYPE:
             # The server sent back a new representation of the object.
             # Use it in preference to the existing representation.
-            new_representation = simplejson.loads(content)
+            new_representation = simplejson.loads(unicode(content))
             self._wadl_resource.representation = new_representation
             self._wadl_resource.media_type = content_type
 
@@ -652,7 +687,7 @@ class Collection(Resource):
             if next_link is None:
                 break
             current_page = simplejson.loads(
-                self._root._browser.get(URI(next_link)))
+                unicode(self._root._browser.get(URI(next_link))))
 
     def __getitem__(self, key):
         """Look up a slice, or a subordinate resource by index.
@@ -725,7 +760,7 @@ class Collection(Resource):
         # Iterate over pages until we have the correct number of entries.
         while more_needed > 0 and page_url is not None:
             representation = simplejson.loads(
-                self._root._browser.get(page_url))
+                unicode(self._root._browser.get(page_url)))
             current_page_entries = representation['entries']
             entry_dicts += current_page_entries[:more_needed]
             more_needed = desired_size - len(entry_dicts)
@@ -817,7 +852,8 @@ class CollectionWithKeyBasedLookup(Collection):
         # is to retrieve a representation of the resource and see how
         # the resource describes itself.
         try:
-            representation = simplejson.loads(self._root._browser.get(url))
+            representation = simplejson.loads(
+                unicode(self._root._browser.get(url)))
         except HTTPError, error:
             # There's no resource corresponding to the given ID.
             if error.response.status == 404:
